@@ -2,6 +2,10 @@ lambdanum <- function(model){
 	length(model$rho)
 }
 
+getNZero <- function(model){
+	model$nzero
+}
+
 updateCoefs <- function(obj,mu_x,sigma_x){
   transform_coefs(obj$coefficients,mu_x,sigma_x,TRUE)
 }
@@ -10,13 +14,18 @@ quick.predict <- function(coefs,newx){
   cbind(1,newx) %*% coefs
 }
 
-predict.models <- function(object, newx){
+predModels <- function(object, newx){
   cbind(1,newx) %*% object$coefficients
 }
 
-predict.errors <- function(object, newx, newy){
-  preds <- predict(object,newx)
+predErrors <- function(object, newx, newy){
+  preds <- cbind(1,newx) %*% coefficients(object)
   errors <- lapply(preds,subtract,newy)
+  errors
+}
+
+checkErrors <- function(object,newx,newy){
+  lapply(object$models,check.errors.model,newx,newy)
 }
 
 check.errors <- function(object,newx,newy){
@@ -24,7 +33,7 @@ check.errors <- function(object,newx,newy){
 }
 
 check.errors.model <- function(object,newx,newy){
-  preds <- predict.models(object,newx)
+  preds <- predModels(object,newx)
   errors <- newy- preds
   check(errors,object$tau)
 }
@@ -46,7 +55,7 @@ model_eval <- function(model, test_x, test_y, test_w=NULL, func="check",...){
 qbic <- function(model, largeP=FALSE){
   tau <- model$tau
   n <- model$n
-  nzero <- apply(model$coefficients !=0,2,sum)
+  nzero <- sum(model$coefficients !=0)
   if(largeP){
     bic <- log(model$rho) + nzero*log(n)*log(length(model$coefficients))/(2*n)
   }else{
@@ -254,12 +263,15 @@ neg.gradient <- function(r,weights,tau,gamma,x,apprx){
 #returns a lambda value that (likely) produces a completely sparse model. 
 #Only likely and not guaranteed, because it relies on huber approximation to the quantile loss
 #Code improvement: potential hacky code for selection of gamma
-getLamMax <- function(x,y,tau=.5,penalty="LASSO",scalex=TRUE,a=NULL,tau.penalty.factor=NULL,penalty.factor=NULL){
+getLamMax <- function(x,y,tau=.5,penalty="LASSO",scalex=TRUE,a=NULL,tau.penalty.factor=NULL,penalty.factor=NULL,weights=NULL){
 	n <- length(y)
 	returnVal <- 0
 	if(scalex){
 		x <- scale(x)
 	}
+	if(is.null(weights)){
+		weights <- rep(1,n)
+	}
 	
 	#to-do: remove for loop
 	if(penalty!="ENet"){
@@ -267,36 +279,36 @@ getLamMax <- function(x,y,tau=.5,penalty="LASSO",scalex=TRUE,a=NULL,tau.penalty.
 	    a <- 1
 	}
 	for(aval in a){
-	  tspot <- 1
-	  if(aval == 0){
-	    aval <- min(1/1000,a[which(a!=0)])
-	  }
-  	for(tau_val in tau){
-  	  pf <- penalty.factor*tau.penalty.factor[tspot]*aval
-  	  validspots <- which(pf!=0)
-  	  pf <- pf[validspots]
-  	  npenVars <- which(!(1:ncol(x) %in% validspots))
-  	  
-  	  if(length(npenVars)==0){
-  	    # intercept 
-  	    b.int<- quantile(y, probs = tau_val)
-  	    r<- y-b.int
-  	  } else{
-  	    q1 <- rq(y~x[,npenVars],tau=tau_val)
-  	    r <- resid(q1)
-  	  }
-  		if(abs(1-tau_val)<.05){
-  		  gamma.q <- .01
-  		  gamma.max <- .0001
-  		} else{
-  		  gamma.q <- .1
-  		  gamma.max <- .001
-  		}
-  		gamma0<- max(gamma.max, quantile(abs(r), probs = gamma.q))
-  		grad_k<- -neg.gradient(r, rep(1,n), tau_val, gamma=gamma0, x, apprx="huber")[validspots]
-  		returnVal <- max(c(returnVal,abs(grad_k)/pf))
-  		tspot <- tspot + 1
-  	}
+		tspot <- 1
+		if(aval == 0){
+			aval <- min(1/1000,a[which(a!=0)])
+		}
+		for(tau_val in tau){
+			pf <- penalty.factor*tau.penalty.factor[tspot]*aval
+			validspots <- which(pf!=0)
+			pf <- pf[validspots]
+			npenVars <- which(!(1:ncol(x) %in% validspots))
+		  
+			if(length(npenVars)==0){
+			# intercept 
+				q1 <- rq(y~1,weights=weights,tau=tau_val)
+			} else{
+				q1 <- rq(y~x[,npenVars],weights=weights,tau=tau_val) 	    
+			}
+			r	<- resid(q1)
+			
+			if(abs(1-tau_val)<.05){
+			  gamma.q <- .01
+			  gamma.max <- .0001
+			} else{
+			  gamma.q <- .1
+			  gamma.max <- .001
+			}
+			gamma0<- max(gamma.max, quantile(abs(r), probs = gamma.q))
+			grad_k<- -neg.gradient(r, weights, tau_val, gamma=gamma0, x, apprx="huber")[validspots]
+			returnVal <- max(c(returnVal,abs(grad_k)/pf))
+			tspot <- tspot + 1
+		}
 	}
 	returnVal*1.05
 }
@@ -310,13 +322,16 @@ l1norm <- function(x){
 }
 
 # Finds lambda max for a group penalty. 
-getLamMaxGroup <- function(x,y,group.index,tau=.5,group.pen.factor,gamma=.2,gamma.max=4,gamma.q=.1,penalty="gLASSO",scalex=TRUE,tau.penalty.factor,norm=2){
+getLamMaxGroup <- function(x,y,group.index,tau=.5,group.pen.factor,gamma=.2,gamma.max=4,gamma.q=.1,penalty="gLASSO",scalex=TRUE,tau.penalty.factor,norm=2,weights=NULL){
 # code improvement: Hacky approach to the group.pen.factor issue. 
   lambda.max <- 0
 	n <- length(y)
 	if(scalex){
 		x <- scale(x)
 	}
+	if(is.null(weights)){
+		weights <- rep(1,n)
+	}
 	i <- 1
 	for(tau_val in tau){
 	  pen.factor <- group.pen.factor*tau.penalty.factor[i]
@@ -325,15 +340,14 @@ getLamMaxGroup <- function(x,y,group.index,tau=.5,group.pen.factor,gamma=.2,gamm
 	  
 	  if(length(npenVars)==0){
 	    # intercept 
-	    b.int<- quantile(y, probs = tau_val)
-	    r<- y-b.int
+	    q1 <- rq(y ~ 1, tau=tau_val,weights=weights)
 	  } else{
-	    q1 <- rq(y~x[,npenVars],tau=tau_val)
-	    r <- resid(q1)
+	    q1 <- rq(y~x[,npenVars],tau=tau_val,weights=weights)
 	  }
+	  r <- resid(q1)
 		gamma0<- min(gamma.max, max(gamma, quantile(abs(r), probs = gamma.q)))
 
-		grad_k<- -neg.gradient(r, rep(1,n), tau_val, gamma=gamma0, x, apprx="huber")
+		grad_k<- -neg.gradient(r, weights, tau_val, gamma=gamma0, x, apprx="huber")
 		if(norm==2){
 		  grad_k.norm<- tapply(grad_k, group.index, l2norm)
 		}
@@ -352,7 +366,7 @@ getLamMaxGroup <- function(x,y,group.index,tau=.5,group.pen.factor,gamma=.2,gamm
 # lambda.discard not used
 rq.lasso <- function(x,y,tau=.5,lambda=NULL,nlambda=100,eps=ifelse(nrow(x)<ncol(x),.01,.0001), penalty.factor = rep(1, ncol(x)),
 						alg=c("huber","br"),scalex=TRUE,tau.penalty.factor=rep(1,length(tau)),
-						coef.cutoff=1e-8,max.iter=10000,converge.eps=1e-7,lambda.discard=TRUE,...){
+						coef.cutoff=1e-8,max.iter=10000,converge.eps=1e-7,lambda.discard=TRUE,weights=NULL,...){
 	if(alg == "lp"){
 	#use br as the default for linear programming 
 		alg <- "br"
@@ -384,7 +398,11 @@ rq.lasso <- function(x,y,tau=.5,lambda=NULL,nlambda=100,eps=ifelse(nrow(x)<ncol(
 		if(length(lambda)==1){
 			stop("The Huber algorithm requires at least 2 values of lambda")
 		}
-		returnVal <- rq.lasso.huber(x,y,tau,lambda,penalty.factor,scalex,tau.penalty.factor,max.iter,converge.eps,lambda.discard=lambda.discard,...)
+		if(is.null(weights)){
+			returnVal <- rq.lasso.huber(x,y,tau,lambda,penalty.factor,scalex,tau.penalty.factor,max.iter,converge.eps,lambda.discard=lambda.discard,...)
+		} else{
+			returnVal <- rq.glasso(x,y,tau,1:p, lambda,penalty.factor,scalex,tau.penalty.factor,max.iter,converge.eps,lambda.discard=lambda.discard,weights=weights,gamma=.2,...)
+		}
 	} else{
 		models <- vector(mode="list",length=nt)
 		modelnames <- NULL
@@ -393,11 +411,11 @@ rq.lasso <- function(x,y,tau=.5,lambda=NULL,nlambda=100,eps=ifelse(nrow(x)<ncol(
 			j <- 1
 			for(lam in lambda){
 				sublam <- lam*penalty.factor*tau.penalty.factor[i]
-				subm <- rq.lasso.fit(x,y,tau[i],lambda=sublam, method=alg,scalex=scalex, coef.cutoff=coef.cutoff,lambda.discard=lambda.discard,...)
+				subm <- rq.lasso.fit(x,y,tau[i],lambda=sublam, method=alg,scalex=scalex, coef.cutoff=coef.cutoff,lambda.discard=lambda.discard,weights=weights,...)
 				coefs <- cbind(coefs,coefficients(subm))
 				j <- j + 1
 			}
-			models[[i]] <- rq.pen.modelreturn(coefs,x,y,tau[i],lambda,penalty.factor*tau.penalty.factor[i],"LASSO",1)
+			models[[i]] <- rq.pen.modelreturn(coefs,x,y,tau[i],lambda,penalty.factor*tau.penalty.factor[i],"LASSO",1,weights=weights)
 			modelnames <- c(modelnames,paste0("tau",tau[i],"a",1))
 		}
 		names(models) <- modelnames
@@ -470,7 +488,7 @@ rq.enet <- function(x,y,tau=.5,lambda=NULL,nlambda=100,eps=ifelse(nrow(x)<ncol(x
 
 #kind of hacky code with respect to rhoPen calculation 
 rq.lla <- function(obj,x,y,penalty,a=ifelse(penalty=="SCAD",3.7,3),penalty.factor,tau.penalty.factor,scalex=TRUE,
-					coef.cutoff=1e-8,max.iter=10000,converge.eps=1e-7,lambda.discard=TRUE,...){
+					coef.cutoff=1e-8,max.iter=10000,converge.eps=1e-7,lambda.discard=TRUE,weights=NULL,...){
 	nt <- length(obj$tau)
 	na <- length(a)
 	if(penalty=="SCAD"){
@@ -501,7 +519,7 @@ rq.lla <- function(obj,x,y,penalty,a=ifelse(penalty=="SCAD",3.7,3),penalty.facto
 				llapenf <- derivf(as.numeric(abs(coefficients(obj$models[[j]]))[-1,i]),lampen*obj$lambda[i],a=a[k])
 				if(sum(llapenf)==0){
 					if(!endHit){
-						update_est <- try(coefficients(rq(y~x,tau=obj$tau[j])),silent=TRUE)
+						update_est <- try(coefficients(rq(y~x,tau=obj$tau[j],weights=weights)),silent=TRUE)
 						if(inherits(update_est,"try-error")){
 							warning(paste0("At lambda value ", obj$lambda[i], " method is equivalent to an unpenalized method but quantreg:::rq(y~x) fails at quantile", obj$tau[j], ". Thus method stops at lambda", obj$lambda[i-1]))
 							i <- i -1
@@ -512,10 +530,10 @@ rq.lla <- function(obj,x,y,penalty,a=ifelse(penalty=="SCAD",3.7,3),penalty.facto
 					}
 				}
 				else if(obj$alg=="huber"){
-					update_est <- coefficients(rq.lasso(x,y,obj$tau[j],lambda=c(2,1),penalty.factor=llapenf,scalex=scalex,alg=obj$alg,coef.cutoff=coef.cutoff,max.iter=max.iter,converge.eps=converge.eps,lambda.discard=FALSE,...)$models[[1]])[,2]
+					update_est <- coefficients(rq.lasso(x,y,obj$tau[j],lambda=c(2,1),penalty.factor=llapenf,scalex=scalex,alg=obj$alg,coef.cutoff=coef.cutoff,max.iter=max.iter,converge.eps=converge.eps,lambda.discard=FALSE,weights=weights,...)$models[[1]])[,2]
 
 				} else{
-					update_est <- coefficients(rq.lasso(x,y,obj$tau[j],lambda=1,penalty.factor=llapenf,alg=obj$alg,scalex=scalex,coef.cutoff=coef.cutoff,max.iter=max.iter,converge.eps=converge.eps,lambda.discard=FALSE,...)$models[[1]])
+					update_est <- coefficients(rq.lasso(x,y,obj$tau[j],lambda=1,penalty.factor=llapenf,alg=obj$alg,scalex=scalex,coef.cutoff=coef.cutoff,max.iter=max.iter,converge.eps=converge.eps,lambda.discard=FALSE,weights=weights,...)$models[[1]])
 				}
 				newModels[[pos]]$coefficients[,i] <- update_est
 				if(penalty=="aLASSO"){
@@ -530,7 +548,7 @@ rq.lla <- function(obj,x,y,penalty,a=ifelse(penalty=="SCAD",3.7,3),penalty.facto
 				}
 			}
 			newModels[[pos]]$a <- a[k]
-			newModels[[pos]] <- rq.pen.modelreturn(newModels[[pos]]$coefficients,x,y,newModels[[pos]]$tau,obj$lambda,local.penalty.factor=penalty.factor*tau.penalty.factor[j],penalty,a[k])
+			newModels[[pos]] <- rq.pen.modelreturn(newModels[[pos]]$coefficients,x,y,newModels[[pos]]$tau,obj$lambda,local.penalty.factor=penalty.factor*tau.penalty.factor[j],penalty,a[k],weights=weights)
 			newModels[[pos]]$PenRho <- penSums + newModels[[pos]]$rho
 			modelNames <- c(modelNames,paste0("tau",obj$tau[j],"a",a[k]))
 			pos <- pos+1
@@ -558,7 +576,7 @@ clearModels <- function(model,pos){
 
 
 rq.group.lla <- function(obj,x,y,groups,penalty=c("gAdLASSO","gSCAD","gMCP"),a=NULL,norm=2, group.pen.factor,
-						tau.penalty.factor,scalex,coef.cutoff,max.iter,converge.eps,gamma,lambda.discard,...){
+						tau.penalty.factor,scalex,coef.cutoff,max.iter,converge.eps,gamma,lambda.discard,weights,...){
 	#for loop calculation of penalty factors that could maybe be removed
 	nt <- length(obj$tau)
 	p <- ncol(x)
@@ -590,7 +608,7 @@ rq.group.lla <- function(obj,x,y,groups,penalty=c("gAdLASSO","gSCAD","gMCP"),a=N
 				#print(paste("derivs are ", coef_by_group_deriv))
 				if(sum(coef_by_group_deriv)==0){
 					if(n > p + 1){
-						update_est <- try(coefficients(rq(y~x,tau=obj$tau[j])), silent=TRUE)
+						update_est <- try(coefficients(rq(y~x,weights=weights,tau=obj$tau[j])), silent=TRUE)
 						if(inherits(update_est,"try-error")){
 						  warning(paste0("At lambda value ", obj$lambda[i], " method is equivalent to an unpenalized method but quantreg:::rq(y~x) fails at quantile", obj$tau[j], ". Thus method stops at lambda", obj$lambda[i-1]))
 						  i <- i -1
@@ -613,14 +631,14 @@ rq.group.lla <- function(obj,x,y,groups,penalty=c("gAdLASSO","gSCAD","gMCP"),a=N
 					if(obj$alg=="huber"){
 						if(norm == 1){
 							penalty.factor <- mapvalues(groups,seq(1,g),coef_by_group_deriv)
-							update_est <- coefficients(rq.lasso(x,y,obj$tau[j],lambda=c(2,1),penalty.factor=penalty.factor,alg=obj$alg,scalex=scalex,coef.cutoff=coef.cutoff,max.iter=max.iter,converge.eps=converge.eps,lambda.discard=FALSE,...)$models[[1]])[,2]
+							update_est <- coefficients(rq.lasso(x,y,obj$tau[j],lambda=c(2,1),penalty.factor=penalty.factor,alg=obj$alg,scalex=scalex,coef.cutoff=coef.cutoff,max.iter=max.iter,converge.eps=converge.eps,lambda.discard=FALSE,weights=weights,...)$models[[1]])[,2]
 						} else{
-							update_est <- coefficients(rq.group.pen(x,y,obj$tau[j],groups,lambda=1,group.pen.factor=coef_by_group_deriv, alg=obj$alg,scalex=scalex,coef.cutoff=coef.cutoff,max.iter=max.iter,converge.eps=converge.eps,gamma=gamma,lambda.discard=FALSE,...)$models[[1]])
+							update_est <- coefficients(rq.group.pen(x,y,obj$tau[j],groups,lambda=1,group.pen.factor=coef_by_group_deriv, alg=obj$alg,scalex=scalex,coef.cutoff=coef.cutoff,max.iter=max.iter,converge.eps=converge.eps,gamma=gamma,lambda.discard=FALSE,weights=weights,...)$models[[1]])
 						}
 					} else{
 					#norm must be 1 as huber is only valid approach for norm 2
 						penalty.factor <- mapvalues(groups,seq(1,g),coef_by_group_deriv)
-						update_est <- coefficients(rq.lasso(x,y,obj$tau[j],lambda=1,penalty.factor=penalty.factor,alg=obj$alg,scalex=scalex,coef.cutoff=coef.cutoff,max.iter=max.iter,converge.eps=converge.eps,lambda.discard=FALSE,...)$models[[1]])
+						update_est <- coefficients(rq.lasso(x,y,obj$tau[j],lambda=1,penalty.factor=penalty.factor,alg=obj$alg,scalex=scalex,coef.cutoff=coef.cutoff,max.iter=max.iter,converge.eps=converge.eps,lambda.discard=FALSE,weights=weights,...)$models[[1]])
 					}
 				}
 				newModels[[pos]]$coefficients[,i] <- update_est
@@ -639,7 +657,7 @@ rq.group.lla <- function(obj,x,y,groups,penalty=c("gAdLASSO","gSCAD","gMCP"),a=N
 				newModels[[pos]] <- clearModels(newModels[[pos]],i)
 			}
 			
-			newModels[[pos]] <- rq.pen.modelreturn(newModels[[pos]]$coefficients,x,y,obj$tau[j],obj$lambda[1:i],rep(1,p),penalty,a[k])	
+			newModels[[pos]] <- rq.pen.modelreturn(newModels[[pos]]$coefficients,x,y,obj$tau[j],obj$lambda[1:i],rep(1,p),penalty,a[k],weights=weights)	
 			newModels[[pos]]$penalty.factor <- NULL	
 			if(penalty=="gAdLASSO"){
 				newModels[[pos]]$PenRho <- newModels[[pos]]$rho + penVals
@@ -735,7 +753,7 @@ createModelsInfo <- function(models){
 }
 
 rq.nc <- function(x, y, tau=.5,  penalty=c("SCAD","aLASSO","MCP"),a=NULL,lambda=NULL,nlambda=100,eps=ifelse(nrow(x)<ncol(x),.01,.0001),alg=c("huber","br","QICD"),scalex=TRUE,
-					penalty.factor = rep(1, ncol(x)),tau.penalty.factor=rep(1,length(tau)),coef.cutoff=1e-8,max.iter=10000,converge.eps=1e-7,lambda.discard=TRUE,...) {
+					penalty.factor = rep(1, ncol(x)),tau.penalty.factor=rep(1,length(tau)),coef.cutoff=1e-8,max.iter=10000,converge.eps=1e-7,lambda.discard=TRUE,weights=NULL,...) {
 	#should look at how ncvreg generates the lambda sequence and combine that with the Huber based approach
 	penalty <- match.arg(penalty)
 	nt <- length(tau)
@@ -764,9 +782,9 @@ rq.nc <- function(x, y, tau=.5,  penalty=c("SCAD","aLASSO","MCP"),a=NULL,lambda=
 		} else{
 			init.model <- rq.lasso(x,y,tau,alg=alg,lambda=lambda,scalex=scalex,penalty.factor=penalty.factor,
 						tau.penalty.factor=tau.penalty.factor,coef.cutoff=coef.cutoff,max.iter=max.iter,
-						converge.eps=converge.eps,lambda.discard=lambda.discard,...)
+						converge.eps=converge.eps,lambda.discard=lambda.discard,weights=weights,...)
 		}
-		rq.lla(init.model,x,y,penalty,a,penalty.factor,tau.penalty.factor,scalex,coef.cutoff,max.iter,converge.eps,lambda.discard=lambda.discard,...)
+		rq.lla(init.model,x,y,penalty,a,penalty.factor,tau.penalty.factor,scalex,coef.cutoff,max.iter,converge.eps,lambda.discard=lambda.discard,weights=weights,...)
 	} else{
 		if(length(unique(penalty.factor))!=1 & length(unique(tau.penalty.factor))!=1){
 			warning("The QICD algorithm takes predictor and tau penalty factors and turns them into a zero or 1. Zero if the weight is zero and one otherwise. Other algorithms are better if you want a more nuanced approach.")
@@ -780,10 +798,10 @@ rq.nc <- function(x, y, tau=.5,  penalty=c("SCAD","aLASSO","MCP"),a=NULL,lambda=
 				for(lam in lambda){
 					sublam <- lam*penalty.factor*tau.penalty.factor[i]
 					penvars <- which(sublam != 0)
-					subm <- rq.nc.fit(x,y,tau[i],lambda=lam, alg="QICD", a=a[j], coef.cutoff=coef.cutoff,converge_criteria=converge.eps,penVars=penvars,internal=TRUE,...)
+					subm <- rq.nc.fit(x,y,tau[i],lambda=lam, alg="QICD", a=a[j], coef.cutoff=coef.cutoff,converge_criteria=converge.eps,penVars=penvars,internal=TRUE,weights=weights,...)
 					coefs <- cbind(coefs,coefficients(subm))
 				}
-				models[[pos]] <- rq.pen.modelreturn(coefs,x,y,tau[i],lambda,local.penalty.factor=penalty.factor*tau.penalty.factor[j],penalty,a[j])
+				models[[pos]] <- rq.pen.modelreturn(coefs,x,y,tau[i],lambda,local.penalty.factor=penalty.factor*tau.penalty.factor[j],penalty,a[j],weights=weights)
 				modelNames <- c(modelNames,paste0("tau",tau[i],"a",a[j]))
 				pos <- pos + 1
 			}
@@ -827,7 +845,7 @@ getDerivF <- function(penalty){
 	derivf
 }
 
-rq.glasso <- function(x,y,tau,groups, lambda, group.pen.factor,scalex,tau.penalty.factor,max.iter,converge.eps,gamma,lambda.discard,...){
+rq.glasso <- function(x,y,tau,groups, lambda, group.pen.factor,scalex,tau.penalty.factor,max.iter,converge.eps,gamma,lambda.discard,weights,...){
 	dims <- dim(x)
 	n <- dims[1]
 	p <- dims[2]
@@ -839,8 +857,8 @@ rq.glasso <- function(x,y,tau,groups, lambda, group.pen.factor,scalex,tau.penalt
 	for(i in 1:nt){
 		subtau <- tau[i]
 		penf <- group.pen.factor*tau.penalty.factor[i]
-		models[[i]] <- hrq_glasso(x,y,group.index=groups,tau=subtau,lambda=lambda,w.lambda=penf,scalex=scalex,gamma=gamma,max_iter=max.iter,epsilon=converge.eps,lambda.discard=lambda.discard,...)
-		models[[i]] <- rq.pen.modelreturn(models[[i]]$beta,x,y,subtau,models[[i]]$lambda,rep(1,p),"gLASSO",1)
+		models[[i]] <- hrq_glasso(x,y,group.index=groups,tau=subtau,lambda=lambda,w.lambda=penf,scalex=scalex,gamma=gamma,max_iter=max.iter,epsilon=converge.eps,lambda.discard=lambda.discard,weights=weights,...)
+		models[[i]] <- rq.pen.modelreturn(models[[i]]$beta,x,y,subtau,models[[i]]$lambda,rep(1,p),"gLASSO",1,weights=weights)
 		#min_n_coefs <- min(min_n_coefs)
 	}
 	attributes(models)$names <- paste0("tau",tau,"a",1)
@@ -881,13 +899,17 @@ qaSIS <- function(x,y,tau=.5,linear=FALSE,...){#n.cores=1,...){
 
 
 #need to update this and think about what penalty.factor needs to be used. 
-rq.pen.modelreturn <- function(coefs,x,y,tau,lambda,local.penalty.factor,penalty,a){
+rq.pen.modelreturn <- function(coefs,x,y,tau,lambda,local.penalty.factor,penalty,a,weights=NULL){
 # for loop that could be removed
 	penfunc <- getPenfunc(penalty)
 	return_val <- NULL
 	if(is.null(ncol(coefs))==FALSE){
 		colnames(coefs) <- paste0("L",1:ncol(coefs))
 	}
+	n <- length(y)
+	if(is.null(weights)){
+		weights <- rep(1,n)
+	}
 	return_val$coefficients <- coefs
 	#return_val$penalty.factor <- penalty.factor
 	p <- ncol(x)
@@ -903,11 +925,11 @@ rq.pen.modelreturn <- function(coefs,x,y,tau,lambda,local.penalty.factor,penalty
 	#return_val$fitted <- fits
 	res <- y - fits
 	if(is.null(dim(return_val$coefficients))==TRUE){
-		return_val$rho <- mean(check(res,tau))	
+		return_val$rho <- mean(check(res,tau)*weights)	
 		return_val$PenRho <- return_val$rho + sum(penfunc(return_val$coefficients[-1],lambda*local.penalty.factor,a))
 		return_val$nzero <- sum(return_val$coefficients!=0)
 	} else{
-		return_val$rho <- apply(check(res,tau),2,mean)	
+		return_val$rho <- apply(check(res,tau)*weights,2,mean)	
 		rownames(return_val$coefficients) <- x_names
 		for(i in 1:length(return_val$rho)){
 			return_val$PenRho[i] <- return_val$rho[i] + sum(penfunc(return_val$coefficients[-1,i],lambda[i]*local.penalty.factor,a))
